/*
*************************************************

		CUT

Cut (select) part of a text file.

     1. A starting line number and ending line
	number can be specified. If none is
	specified, then the beginning and/or end
	of the file is assumed.

    2. A starting column and ending column can
       be specified. If the ending column is
       beyond the length of a line, then the
       end of the line is used. If no starting
       and/or ending column is specified, then
       the first and/or last column is used.


    Starting and ending values must fit in an int,
    which is usually less than 32K.

    Text lines must be no more than 255 columns.


Command line arguments are:
    -cs start column	 -  first column if not specifed or invalid value
    -ce end   column	 -  line length if not specified or invalid value
    -ls start line	 -  first line of not specified or invalid value
    -le end   line	 -  EOF if not specified or invalid value
    -i	input file name  -  STDIN if not specified
    -o	output file name -  STDOUT if not specified
    -s	delete trailing spaces
    -p	padd lines to specified column length (-ce + -cs + 1) with spaces
    -b	skip lines that begin with a space (prior to -cs test)
    -n	skip null lines (determined prior to -cs test)

Example command lines:

    cut
    cut -cs1 -ce8 -iintext.txt -oouttext.txt
    cut -cs1 -ce8 -ls4 -iintext.txt -oouttext.txt

*************************************************

     3 April 1985
     Dean A. Roth
     Milwaukee, WI

*************************************************
*/

#include <stdio.h>

#define DEFAULT -1
#define EOFERR ((FILE *) NULL)	 /* End Of File or an error */
#define PASS 1
#define FAIL 0







main( argc, argv)

    int  argc;
    char *argv[];

{
    FILE *fopen();
    char *fgets();
    int  atoi();



    int     cstart = 1; 	 /* start column - default is first column */
    int     cend   = DEFAULT;	 /* end column	 - default is last column  */

    int     lstart = 1; 	 /* start line - default is end of file */
    int     lend   = DEFAULT;	 /* end line   - default is end of file */

    int     line;		 /* current line read from file */

    int     loop;		 /* misc loop counter */


    FILE    *fpin  = EOFERR;	 /* input file handle  - default is STDIN  */
    FILE    *fpout = EOFERR;	 /* output file handle - default is STDOUT */


    int     strip  = FAIL;	 /* default is do not delete trailing spaces */
    int     padd   = FAIL;	 /* default is do not padd lines */
    int     snull  = FAIL;	 /* default is do not skip null lines */
    int     sblank = FAIL;	 /* default is do not skip lines that begin with a space */


    char    buffer[1024];


    /* DECOMPOSE COMMAND LINE */
    for (loop = 1; loop < argc; loop++)     /* skip first arg, which is prog name */
    {
	strtoupper(argv[loop]);               /* convert to uppercase */

	/* all options begin with a "-" */
	if (argv[loop][0] != '-')
	    fprintf(stderr, "\nInvalid option:%s" );
	else
	    switch (argv[loop][1])
	    {
	    case    'C' : /* column starting & ending positions */
		switch (argv[loop][2])
		{
		case	'S' : /* starting column - must be at least one (1) */
		    cstart = atoi(&argv[loop][3]);
		    if (cstart < 1 || cstart > 255)
		    {
			fprintf(stderr,"\nInvalid start column:%d. Must be >= 1 and <= 255.", cstart);
			cuthelp();
			exit(1);
		    }
		    break;


		case	'E' :
		    cend = atoi(&argv[loop][3]);
		    if (cend < 1 || cend > 255)
		    {
			fprintf(stderr,"\nInvalid end column:%d. Must be >= 1 and <= 255.", cend);
			cuthelp();
			exit(1);
		    }
		    break;


		default     : /* ignore invalid arguments */
		    fprintf(stderr,"\nInvalid column option:%s. Processing aborted.", argv[loop]);
		    cuthelp();
		    exit(1);

		}

		break;



	    case    'L' :   /* line start & end values */
		switch (argv[loop][2])
		{
		case	'S' :
		    lstart = atoi(&argv[loop][3]);
		    if (lstart < 1)
		    {
			fprintf(stderr,"\nInvalid start line:%d. Must be >= 1 and <= 32K.", lstart);
			cuthelp();
			exit(1);
		    }
		    break;


		case	'E' :
		    lend = atoi(&argv[loop][3]);
		    if (lend < 1)
		    {
			fprintf(stderr,"\nInvalid end line:%d. Must be >= 1 and <= 32K.", lend);
			cuthelp();
			exit(1);
		    }
		    break;

		default     : /* ignore invalid arguments */
		    fprintf(stderr,"\nInvalid line option:%s. Processing aborted.", argv[loop]);
		    cuthelp();
		    exit(1);

		}

		break;




	    case    'I' :   /* input file name */
		fpin = fopen(&argv[loop][2], "r");
		if (fpin == EOFERR)
		{
		    fprintf(stderr, "\nOpen failed on input file:%s. Processing aborted.", &argv[loop][2]);
		    cuthelp();
		    exit(1);
		}

		break;





	    case    'O' :   /* output file */
		fpout = fopen(&argv[loop][2], "w");
		if (fpout == EOFERR)
		{
		    fprintf(stderr, "\nOpen failed on output file:%s. Processing aborted.", &argv[loop][2]);
		    cuthelp();
		    exit(1);
		}

		break;




	    case 'S' : /* delete trailing spaces */
		strip  = PASS;
		break;



	    case 'P' : /* padd lines with spaces when line is less than specified length (end - start + 1) */
		padd = PASS;
		break;


	    case '?' :
	    case 'H' :  /* help */
		cuthelp();
		exit(0);


	    case 'B' :  /* skip lines that begin with a space */
		sblank = PASS;
		break;


	    case 'N' :  /* skip null lines */
		snull = PASS;
		break;

	    default	: /* skip invalid "-" arguments - NOT -C OR -L */
		fprintf(stderr, "\nInvalid option:%s. Processing aborted.", argv[loop]);
		cuthelp();
		exit(1);
	    }

    }	/* end FOR loop through all command line arguments */



    /* NOW TEST VALIDITY OF ARGUMENTS AGAIN EACH OTHER	AND  SET DEFAULT */
    if (fpin == EOFERR)
	fpin = stdin;

    if (fpout == EOFERR)
	fpout = stdout;


    if (cend != DEFAULT)
	if (cend < cstart)
	{
	    fprintf(stderr, "\nEnd column must be >= to start column. Processing aborted.");
	    cuthelp();
	    exit(1);
	}

    if (lend != DEFAULT)
	if (lend < lstart)
	{
	    fprintf(stderr, "\nEnd line must be >= to start line. Processing aborted.");
	    cuthelp();
	    exit(1);
	}

    if (padd == PASS)
	if (strip == PASS)
	{
	    fprintf(stderr, "\n\"-P and -S are mutually exclusinve.");
	    exit(1);
	}




	/* EVERYTHING IS READY. GET STARTING LINE */
	line = 0;

	while (fgets(buffer, 1024, fpin) != (char *)NULL)
	{
	    /* LOCAL VARIABLES */
	    int   colend;
	    char  linebuf[255 + 1];

	    line++;

	    /*********************/
	    /* GET STARTING LINE */
	    /*********************/
	    if (line < lstart)
		continue;	/* get next line */


	    /*******************************************************/
	    /* STOP AFTER SPECIFIED ENDING LINE HAS BEEN PROCESSED */
	    /*******************************************************/
	    if (lend != DEFAULT)
		if ((line - 1) == lend)
		    break;  /* break out to quit */


	    /* get rid of '\n' char which fgets() passes */
	    buffer[strlen(buffer) - 1] = NULL;


	    /********************/
	    /* SKIP NULL LINES? */
	    /********************/
	    if (snull == PASS)
		if (strlen(buffer) == 0)
		    continue;	/* get next line */


	    /***************************************/
	    /* SKIP LINES THAT BEGIN WITH A SPACE? */
	    /***************************************/
	    if (sblank == PASS)
		if (*buffer == ' ')
		    continue;	/* skip line, go to next line */


	    /*****************/
	    /* GET SUBSTRING */
	    /*****************/
	    if (cend == DEFAULT)
		colend = strlen(buffer);
	    else
		if (cend > strlen(buffer))
		    colend = strlen(buffer);
		else
		    colend = cend;


	    strncpy(linebuf, &buffer[cstart - 1], cend - cstart + 1);


	    /*************************/
	    /* STRIP TRAILING BLANKS */
	    /*************************/
	    if (strip == PASS)
		for (loop = strlen(linebuf); loop > 0; loop--)
		    if (linebuf[loop - 1] != ' ')
			break; /* stop on first non-space char */
		    else
			linebuf[loop - 1] = NULL;  /* delete character */


	    /*****************************/
	    /* PADD WITH TRAILING BLANKS */
	    /*****************************/
	    if (padd == PASS)
		if (strlen(linebuf) < (cend - cstart + 1))
		{
		    for (loop = strlen(linebuf); loop < (cend - cstart + 1); loop++)
			linebuf[loop] = ' ';

		    linebuf[loop] = NULL;
		}



	    /***************/
	    /* OUTPUT LINE */
	    /***************/
	    fprintf(fpout, "%s\n", linebuf);

	}


	exit(0);


}   /* end function */





static
strtoupper(s)

   char *s;

{
    while (*s)
    {
	*s = toupper ((int) *s);
	s++;
    }

}   /* end function */




static
cuthelp ()

{

    fprintf(stderr, "\nCut (select) part of a text file.\n");

    fprintf(stderr, "\nCommand line arguments are:");
    fprintf(stderr, "\n    -cs start column     -  first column if not specifed or invalid value     ");
    fprintf(stderr, "\n    -ce end   column     -  line length if not specified or invalid value     ");
    fprintf(stderr, "\n    -ls start line       -  first line of not specified or invalid value      ");
    fprintf(stderr, "\n    -le end   line       -  EOF if not specified or invalid value             ");
    fprintf(stderr, "\n    -i  input file name  -  STDIN if not specified                            ");
    fprintf(stderr, "\n    -o  output file name -  stderr if not specified                           ");
    fprintf(stderr, "\n    -s  delete trailing spaces                                                ");
    fprintf(stderr, "\n    -p  padd lines to specified column length (-ce + -cs + 1) with spaces   \n");
    fprintf(stderr, "\n    -h  this help screen");
    fprintf(stderr, "\n    -b  skip lines that begin with a space (prior to -cs test) ");
    fprintf(stderr, "\n    -n  skip null lines (determined prior to -cs test)       \n");


    fprintf(stderr, "\n    Starting and ending values must fit in an int,");
    fprintf(stderr, "\n    which is usually less than 32K.\n");

    fprintf(stderr, "\n    Text lines must be no more than 255 columns.");

} /* end function */
